@@ -17,7 +17,7 @@ from _pydevd_bundle._debug_adapter.pydevd_schema import (
     VariablesResponseBody, SetBreakpointsResponseBody, Response,
     Capabilities, PydevdAuthorizeRequest, Request,
     StepInTargetsResponseBody, SetFunctionBreakpointsResponseBody, BreakpointEvent,
-    BreakpointEventBody)
+    BreakpointEventBody, InitializedEvent)
 from _pydevd_bundle.pydevd_api import PyDevdAPI
 from _pydevd_bundle.pydevd_breakpoints import get_exception_class, FunctionBreakpoint
 from _pydevd_bundle.pydevd_comm_constants import (
@@ -163,7 +163,7 @@ class PyDevJsonCommandProcessor(object):
                 return NetCommand(CMD_RETURN, 0, error_response, is_json=True)
 
         else:
-            if DebugInfoHolder.DEBUG_RECORD_SOCKET_READS and DebugInfoHolder.DEBUG_TRACE_LEVEL >= 1:
+            if DebugInfoHolder.DEBUG_TRACE_LEVEL >= 1:
                 pydev_log.info('Process %s: %s\n' % (
                     request.__class__.__name__, json.dumps(request.to_dict(update_ids_to_dap=True), indent=4, sort_keys=True),))
 
@@ -333,6 +333,9 @@ class PyDevJsonCommandProcessor(object):
         terminate_child_processes = args.get('terminateChildProcesses', True)
         self.api.set_terminate_child_processes(py_db, terminate_child_processes)
 
+        terminate_keyboard_interrupt = args.get('onTerminate', 'kill') == 'KeyboardInterrupt'
+        self.api.set_terminate_keyboard_interrupt(py_db, terminate_keyboard_interrupt)
+
         variable_presentation = args.get('variablePresentation', None)
         if isinstance(variable_presentation, dict):
 
@@ -377,6 +380,9 @@ class PyDevJsonCommandProcessor(object):
 
         self.api.set_use_libraries_filter(py_db, self._options.just_my_code)
 
+        if self._options.client_os:
+            self.api.set_ide_os(self._options.client_os)
+
         path_mappings = []
         for pathMapping in args.get('pathMappings', []):
             localRoot = pathMapping.get('localRoot', '')
@@ -493,6 +499,9 @@ class PyDevJsonCommandProcessor(object):
         self.api.set_enable_thread_notifications(py_db, True)
         self._set_debug_options(py_db, request.arguments.kwargs, start_reason=start_reason)
         response = pydevd_base_schema.build_response(request)
+
+        initialized_event = InitializedEvent()
+        py_db.writer.add_command(NetCommand(CMD_RETURN, 0, initialized_event, is_json=True))
         return NetCommand(CMD_RETURN, 0, response, is_json=True)
 
     def on_launch_request(self, py_db, request):
@@ -566,6 +575,16 @@ class PyDevJsonCommandProcessor(object):
         target_id = arguments.targetId
         if target_id is not None:
             thread = pydevd_find_thread_by_id(thread_id)
+            if thread is None:
+                response = Response(
+                    request_seq=request.seq,
+                    success=False,
+                    command=request.command,
+                    message='Unable to find thread from thread_id: %s' % (thread_id,),
+                    body={},
+                )
+                return NetCommand(CMD_RETURN, 0, response, is_json=True)
+
             info = set_additional_thread_info(thread)
             target_id_to_smart_step_into_variant = info.target_id_to_smart_step_into_variant
             if not target_id_to_smart_step_into_variant:
@@ -969,7 +988,18 @@ class PyDevJsonCommandProcessor(object):
         exception_into_arguments = request.arguments
         thread_id = exception_into_arguments.threadId
         max_frames = self._options.max_exception_stack_frames
-        self.api.request_exception_info_json(py_db, request, thread_id, max_frames)
+        thread = pydevd_find_thread_by_id(thread_id)
+        if thread is not None:
+            self.api.request_exception_info_json(py_db, request, thread_id, thread, max_frames)
+        else:
+            response = Response(
+                request_seq=request.seq,
+                success=False,
+                command=request.command,
+                message='Unable to find thread from thread_id: %s' % (thread_id,),
+                body={},
+            )
+            return NetCommand(CMD_RETURN, 0, response, is_json=True)
 
     def on_scopes_request(self, py_db, request):
         '''
